//-----------------------------------------------------------------------------
// Projet               : La physique amusante
// Class                : Noeud.pde
// Auteur               : Sylvain Blocquaux
// Date                 : 16 avril 2011
// Version              : 1.0
//-----------------------------------------------------------------------------

class Noeud {
  PVector position;
  PVector positionPrec;
  PVector selectionRef;
  color couleur = color(255);
  boolean selection = false;
  PVector force = new PVector(0, 0);
  float masse = 0.1; // Masse en kg
  PVector gravitation = new PVector(0, 0.98);
  float friction = 0.001; // Coeff de frottement en kg.s-1 (frottement fluide)
  int taille = 20;

  Noeud (float _masse, PVector _position) {
    masse = _masse;
    position = _position;
    positionPrec = _position;
  }

  void afficher() {
    ellipseMode(CENTER);
    fill(couleur);
    ellipse(position.x, position.y, taille, taille);
  }

  void actualiser(float deltaT) {
    PVector calcul = PVector.mult(position, 2);  // Intégration Verlet x(t+1) = 2 * x(t) - x(t-1) + a * ∆T * ∆T
    PVector vitesse = PVector.div(PVector.sub(position, positionPrec), deltaT);
    calcul.sub(positionPrec);
    PVector acceleration = PVector.div(force, masse); // Loi de Newton : F = M * A
    acceleration.mult(deltaT * deltaT);
    calcul.add(acceleration);
    PVector frottement = PVector.mult(vitesse,(-1) * friction);
    calcul.add(frottement);
    PVector gravite = PVector.mult(gravitation, masse);
    calcul.add(gravite);
    if (PVector.sub(position, calcul).mag() < 0.01) { // Pour permettre l'arret du mobile si la vitesse est très petite
      positionPrec = calcul.get();
    } 
    else {
      positionPrec = position.get();
    }
    position = calcul.get();
    force = new PVector(0, 0);
  }

  void selectionner(PVector PMouse) {
    if (PVector.dist(PMouse, PVector.add(position, new PVector(deplacement, deplacement))) < taille /2) {
      selection = true;
      selectionRef = PVector.sub(position, PMouse);
      couleur = color(100);
    }
  }

  void glisser(PVector PMouse) {
    if (selection) {
      position = PVector.add(selectionRef, PMouse);
    }
  }

  void deSelectionner() {
    selection = false;
    couleur = color(255);
  }
  
  void addForce(PVector _force) {
    force.add(_force);
  }
}

